#include <GL/glfw.h>
#include <FTGL/ftgl.h>
#include <vector>
#include <stdlib.h>
#include <stdio.h>
#include <limits>
#include "polygon.h"

int winWidth = 800, winHeight = 600;
int mouse_x, mouse_y;
Polygon poly;
EdgeList diags;
bool done = false;

void initGraphics();
void GLFWCALL mouseButton(int button, int action);
void GLFWCALL mousePos(int x, int y);
void GLFWCALL keyPress(int key, int action);
void glColor(int c, GLubyte a = 0xff);

int main(int argc, char** argv) {
    initGraphics();
    FTTextureFont font = "FreeSans.ttf";
    font.FaceSize(20);
    char buf[80];

    while (1) {
        if (glfwGetKey(GLFW_KEY_ESC) == GLFW_PRESS || !glfwGetWindowParam(GLFW_OPENED))
            break;

        // render
        glClear(GL_COLOR_BUFFER_BIT);

        poly.render();

        for (int i = 0; i < poly.size(); ++i) {
            sprintf(buf, "%d", i);
            glColor3f(0, 0, 0);
            font.Render(buf, -1, FTPoint(poly[i].x + 6, poly[i].y + 4));
            glColor3f(1, 1, 0);
            font.Render(buf, -1, FTPoint(poly[i].x + 5, poly[i].y + 5));
        }

        if (!done) {
            if (poly.size() > 0) {
                glColor4f(1, 1, 1, .5);
                glBegin(GL_LINE_STRIP);
                glVertex2f(poly.first().x, poly.first().y);
                glVertex2f(mouse_x, mouse_y);
                glVertex2f(poly.last().x, poly.last().y);
                glEnd();
            }
        }

        glColor3f(1, 0, 0);
        glPointSize(12);
        glBegin(GL_POINTS);
        for (int i = 0; i < poly.tp.size(); ++i) {
            glVertex2f(poly.tp[i].x, poly.tp[i].y);
        }
        glEnd();

        glColor3f(0, 0, 1);
        glLineWidth(2);
        glBegin(GL_LINES);
        for (int i = 0; i < poly.tl.size(); ++i) {
            glVertex2f(poly.tl[i].first.x, poly.tl[i].first.y);
            glVertex2f(poly.tl[i].second.x, poly.tl[i].second.y);
        }
        glEnd();

        glColor3f(0, 1, 0);
        glLineWidth(2);
        glBegin(GL_LINES);
        for (int i = 0; i < diags.size(); ++i) {
            glVertex2f(diags[i].first.x, diags[i].first.y);
            glVertex2f(diags[i].second.x, diags[i].second.y);
        }
        glEnd();

        glfwSwapBuffers();
    }
    glfwTerminate();
    return EXIT_SUCCESS;
}

void initGraphics() {
    GLFWvidmode dvm; // desktop video mode

    // initialize glfw
    glfwInit();
    glfwGetDesktopMode(&dvm);
    glfwOpenWindowHint(GLFW_WINDOW_NO_RESIZE, GL_TRUE);
    glfwOpenWindow(winWidth, winHeight, dvm.RedBits, dvm.GreenBits, dvm.BlueBits, 0, 0, 0, GLFW_WINDOW);
    glfwGetWindowSize(&winWidth, &winHeight); // created window may be different than requested size
    glfwSetWindowPos((dvm.Width - winWidth) / 2, (dvm.Height - winHeight) / 2); // center the window
    glfwSetWindowTitle("Convex Decomposition");
    glfwEnable(GLFW_STICKY_KEYS);
    glfwDisable(GLFW_KEY_REPEAT);
    glfwSetMouseButtonCallback(mouseButton);
    glfwSetMousePosCallback(mousePos);
    glfwSetKeyCallback(keyPress);

    // initialize opengl
    glViewport(0, 0, winWidth, winHeight);
    glLineWidth(1.5);
    glPointSize(5);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POINT_SMOOTH);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, winWidth, 0, winHeight); // set origin to bottom left
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0, 0, 0, 0);
    glColor3f(1, 1, 1);
}

void GLFWCALL mouseButton(int button, int action) {
    if (action != GLFW_PRESS)
        return;
    switch (button) {
        case GLFW_MOUSE_BUTTON_LEFT:
            if(!done) poly.push(Point(mouse_x, mouse_y));
            break;
        case GLFW_MOUSE_BUTTON_RIGHT:
            poly.makeCCW();
            diags = poly.decomp();
            done = true;
            break;
    }
}

void GLFWCALL keyPress(int key, int action) {
    if (action != GLFW_PRESS)
        return;
    switch (key) {
        case 'C':
            poly.clear();
            diags.clear();
            done = false;
            break;
    }
}

void GLFWCALL mousePos(int x, int y) {
    mouse_x = x;
    mouse_y = winHeight - y;
}

void glColor(int c, GLubyte a) {
    glColor4ub((c >> 16) & 0xff, (c >> 8) & 0xff, c & 0xff, a);
}
